#!/usr/bin/python3.1
#!C:\Python31\python.exe
"""Snake Steak - A python module to access the Freshmeat JSON API.

Snake Steak can be imported as a python module or run directly from the command
line.

Author: uriel
License: public domain
Homepage: http://werc.cat-v.org/snake_steak/

"""

import sys
import codecs
import json
from http.client import HTTPConnection
from collections import namedtuple

auth_code = 'NO_auth_code_SET' # Maybe this should be None? 
dry_run = False
debug = False

def load_auth_code(auth_code_file=None):
    """Load auth code from auth_code_file, or if None load it from `$HOME/.fm_auth_code`."""
    import os
    if not auth_code_file:
        auth_code_file = os.environ['HOME']+"/.fm_auth_code"
    if os.access(auth_code_file, os.R_OK):
        global auth_code
        auth_code = open(auth_code_file).readline()[:-1]
        _dpr("Loaded auth code: >>%s<<"%auth_code)

# External API
def get_project(name):
    """Get the information for project `name`."""
    j = _jget("/projects/"+name)
    if j:
        return _d2nt('Project', j['project'])

def get_project_releases(name):
    """Get the list of existing releses for project `name`."""
    j = _jget("/projects/%s/releases" % name)
    return [_d2nt('Release', r['release']) for r in j]

def get_project_urls(name):
    """Get the list of urls for project `name`."""
    j = _jget("/projects/%s/urls" % name)
    return [_d2nt('Url', u['url']) for u in j]


def update_url(proj, url_label, new_address):
    """Set the url with `url_label` to point to `new_address``."""
    _dpr(proj, url_label, new_address)
    lnk = None
    urls = get_project_urls(proj)
    d = {"label": url_label, "location": new_address}

    if urls is None:
        return None # Maybe we should throw an exception instead?

    for u in urls:
        if u.label == url_label:
            lnk = u.permalink

    if not lnk:
        # XXX Should instead create it.
        _dpr("WARNING: Update failed, didn't find an url with label %s for project %s" % (url_label, proj))
        return _jpost("/projects/%s/urls" % proj, d, "url", method="POST")

    p = "/projects/%s/urls/%s" % (proj, lnk) 

    return _jpost(p, d, "url", method="PUT")
        

def post_release(proj, version, log, tags=None):
    """Submit a release for version `version` of project `proj` with `log` as short changelog message."""
    if not tags:
        tags = []
    d = {"version":version, "tags":tags, "changelog": log}
    return _jpost("/projects/%s/releases" % proj, d, "release", method="POST")


# Private helper functions
def _jget(path):
    c = HTTPConnection("freshmeat.net:80")
    s = '{"auth_code": "%s"}' % auth_code
        
    c.request("GET", path+".json", s, {"Content-Type": "application/json"})
    r = c.getresponse()
    if r.status == 200:
        return json.loads(codecs.decode(r.read()))
    else:
        print("Request failed: %d %s"%(r.status, r.reason), file=sys.stderr)
        return None # TODO Should raise an exception.

def _jpost(path, obj, name, method="POST"):
    s = json.dumps({"auth_code": auth_code, name: obj})
    path += '.json'
    _dpr(path)
    _dpr(s)

    if not dry_run:
        c = HTTPConnection("freshmeat.net:80")
        c.request(method, path, codecs.encode(s),
            {"Content-Type": "application/json"})
        r = c.getresponse()

        if r.status == 201: # Created (usually for POST)
            return True
        elif r.status == 200: # OK (usually for PUT)
            return True
        else:
            print("%s to %s failed: %d %s" % (method, path, r.status, r.reason), file=sys.stderr)
            return None # TODO Should raise an exception.
    else:
        return True

def _d2nt(name, d, recursive=True):
    """Convert a dict `d` to a named tuple object, return d unchanged if it is not of type dict()."""

    if type(d) != dict:
        return d
    c = namedtuple(name, " ".join(k for k in d)) 
    if recursive:
        d = dict([(k, _d2nt(k, d[k])) for k in d])

    return c(**d)
    
def _dpr(*s):
    """Print s to stderr if debug is True."""
    if debug:
        print (*s, file=sys.stderr)


# Command line interface
def _release(args):
    return post_release(args[0], args[1], sys.stdin.read(), args[2:])

def _show(args):
    p = get_project(args[0])
    if not p:
        return False
    print("Project: ", p.name)
    print("Fm name: ", p.permalink)
    print("Created: ", p.created_at)
    print("Updated: ", p.updated_at)
    print("Short desc: ", p.oneliner)
    print("Tags: ", ", ".join(p.tag_list))
    print("Subscribers: ", p.subscriptions_count)
    urls = get_project_urls(args[0])
    if urls:
        print("Urls:")
        for u in urls:
            print("- %s: %s (%s)" % (u.label, u.host, u.permalink))

def _update(args):
    p = args[0]
    if args[1] == 'url':
        update_url(p, args[2], args[3])

def _usage():
    print("""Usage:
    ss.py [general-options] action [action-arguments]

General options:
    -k fm-key
    -n # Dry run, don't post anything to the server. 

Available actions:
    show project-name
    release project-name version [tags, ...] < changelog_file.txt # Changelog is read from stdin.
    update project-name url url-label new-url
""")

def main():
    """Snake Steak's command line interface."""
    actions = {'release':_release, 'show':_show, 'update':_update}

    load_auth_code()

    args = sys.argv[1:]
    _dpr(args)
    if args and args[0] == '-n':
        global dry_run
        dry_run = True
        global debug
        debug = True
        args = args[1:]

    if args and args[0] == '-k':
        global auth_code
        auth_code = args[1]
        args = args[2:]

    if len(args) < 2 or args[0] not in actions:
        _usage()
        exit(2)

    
    if not actions[args[0]](args[1:]):
        exit(1)

if __name__ == '__main__':
    main()
